import ogg.vorbis
import MP3Info
import EXIF
import mmpython
import gnomevfs, os, string
import thumbnails, config
config = config.Config()
thumbnails = thumbnails.Thumb()

class Info:
    def __init__(self): pass
        
        
    def get(self, file):    
        meta = {}
        img = None
        file_path, file_name, file_type, file_size, file_mime = self.__stat(file)
        if file_mime == "audio/vorbis" or file_mime == "application/ogg":
            meta = self.__vorbis(file)                                        

        elif file_mime == "audio/mpeg":
            meta = self.__mp3(file)
         
        elif file_mime == "image/jpeg":
            meta = self.__image(file)
        
        elif file_mime == "video/x-msvideo" or type == "video/mpeg":
            meta = self.__video(file)
            
        if config.get("thumbnails") == "True": 
            class file_info:
                path = file_path
                name = file_name
                mime = file_mime
                size = file_size
                type = file_type            
            img = thumbnails.get(file_info)
        return file_path, file_name, file_size, file_type, file_mime, meta, img


    def __vorbis(self, file):
        meta = {}
        info = ogg.vorbis.VorbisFile(file)
        vc = info.comment()
        for key, val in vc.items():
            if val != None: meta[key] = val
            
        vi = info.info()
        meta["CHANNELS"] = str(vi.channels)
        meta["RATE"] = str(vi.rate)
        meta["BITRATE"]  = str(vi.bitrate_nominal/1000)
        meta["LENGTH"]  = str(info.time_total(-1))       
        return meta
        
    def __mp3(self, file):
        meta = {}
        keys = ['TITLE', 'ARTIST', 'ALBUM', 'YEAR', 'COMMENT', 'TRACKNUMBER', 'GENRE']
        info = MP3Info.MP3Info(open(file, 'rb'))
        for key in keys:
            if string.lower(key) in  info.__dict__.keys():
                if info.__dict__[string.lower(key)] != None: meta[key] = info.__dict__[string.lower(key)]
        meta["BITRATE"]  = info.mpeg.__dict__["bitrate"]
        meta["LENGTH"]  = info.mpeg.__dict__["total_time"]            
        return meta
        
    def __video(self, file):
        meta = {}
        medium = mmpython.parse(file)
        if medium:
            
            for key in medium.keys:    
                if medium[key] != None and key not in ['url', 'delay'] : meta[key] = medium[key]
            
            if medium.has_key("video"):
                    video = medium.video[0]
                    for key in video.keys: 
                        if video[key] != None :  meta["video_" + key] = video[key]
            
            if medium.has_key("audio"):
                audio = medium.audio[0]
            for key in audio.keys:                 
                if audio[key] != None  :  meta["audio_" + str(key)] = audio[key]       
        return meta
                        
                        
    def __image(self, file):
        meta = {}
        try:
            tags = EXIF.process_file(open(file, 'rb'))
        except: 
            tags = {}
        
        for item in  ("EXIF ApertureValue",  "EXIF BrightnessValue",
                       "EXIF ColorSpace",
                       "EXIF ComponentsConfiguration",
                       "EXIF DateTimeDigitized",
                       "EXIF ExifImageLength",
                       "EXIF ExifImageWidth",
                       "EXIF ExifVersion",
                       "EXIF ExposureBiasValue",
                       "EXIF ExposureProgram",
                       "EXIF ExposureTime",
                       "EXIF FNumber",
                       "EXIF Flash",
                       "EXIF FlashEnergy",
                       "EXIF FlashPixVersion",
                       "EXIF FocalLength",
                       "EXIF ISOSpeedRatings",
                       "EXIF InteroperabilityOffset",
                       "EXIF LightSource",
                       "EXIF MeteringMode",
                       "EXIF ShutterSpeedValue",
                       "EXIF SubjectDistance",
                       "Image Model", "Image Make", "EXIF MaxApertureValue","EXIF MeteringMode", "EXIF SensingMethod", "JPEGThumbnail"):
            if tags.has_key(item):
                if tags[item] != None : meta[item] = tags[item]
        return meta
    
    
    
    
    def __stat(self, file):
        self.cwd = gnomevfs.URI(os.getcwd())
        if str(self.cwd)[-1] != '/': self.cwd = self.cwd.append_string('/')
        file_uri = self.cwd.resolve_relative(file)
        try:
            file_info = gnomevfs.get_file_info(file_uri, gnomevfs.FILE_INFO_GET_MIME_TYPE)
        except:
            class file_info:
                name = file
                file_type = ""
                file_size = 0
                mime = ""
#            print 'Name:      ', file_info.name

        file_type = '(none)'
        try: file_type = ('unknown', 'regular', 'directory',
                          'fifo', 'socket', 'chardev', 'blockdev',
                          'symlink')[file_info.type]
        except: pass
#            print 'Type:      ', file_type

        file_size = '(unknown)'
        try: file_size = file_info.size
        except: pass
#            print 'Size:      ', file_size

        mime_type = '(none)'
        try: mime_type = file_info.mime_type
        except: pass
        #print 'Mime type: ', mime_type
        return file_uri, file_info.name, file_type, file_size, mime_type

    