# -*- coding: UTF-8 -*-
import gtk, os.path
import fileinfo, utils, config

w = h = 20
icon_theme = gtk.icon_theme_get_default()

icons = {
                    "directory" :gtk.gdk.pixbuf_new_from_file_at_size(utils.locate_file("directory.png", "icons"), w, h),
                    "audio" : gtk.gdk.pixbuf_new_from_file_at_size(utils.locate_file("gnome-mime-audio.png", "icons"), w, h),        
                    "video" :gtk.gdk.pixbuf_new_from_file_at_size(utils.locate_file("gnome-mime-video.png", "icons"), w, h),
                    "image" : gtk.gdk.pixbuf_new_from_file_at_size(utils.locate_file("gnome-mime-image.png", "icons"), w, h),
                    "package" : gtk.gdk.pixbuf_new_from_file_at_size(utils.locate_file("gnome-package.png", "icons"), w, h),
                    "default" : gtk.gdk.pixbuf_new_from_file_at_size(utils.locate_file("gnome-mime-text.png", "icons"), w, h),
                    "loading" : gtk.gdk.pixbuf_new_from_file_at_size(utils.locate_file("gnome-fs-loading-icon.png", "icons"), w, h),                   
                    "catalog" : gtk.gdk.pixbuf_new_from_file_at_size(utils.locate_file("catalog.png", "icons"), 26, 26),
                    "cdrom" : gtk.gdk.pixbuf_new_from_file_at_size(utils.locate_file("cdrom.png", "icons"), 20, 20),                                         
                    "star_on" : gtk.gdk.pixbuf_new_from_file_at_size(utils.locate_file("star_set.png", "icons"), 16,16),     
                    "star_off" : gtk.gdk.pixbuf_new_from_file_at_size(utils.locate_file("star_unset.png", "icons"), 16,16)     
                    
        }

config = config.Config()

class FstypeBase:
    
    def __init__(self):
        self._data_loaded = False
        self.db = self.id = self.name = self.comment = None
        self.items = []
        self.size = None
        self.rating = None  
        self._index = 0
        self._count = 0
        self.iter = None
        
    def is_catalog(self): return False
    def is_disk(self): return False
    def is_file(self): return False
    def is_dir(self): return False
    
    def __iter__(self):
        if not self._data_loaded:      
            self.items = self._load_data() 
            self._data_loaded = True
            
        self._count = len(self.items) 
        self._index = 0
        return self
        
    def next(self):
        if self._index >= self._count: raise StopIteration
        self._index = self._index + 1
        return self.items[self._index - 1]
    
    def _load_data(self): pass
    
    

class Catalog(FstypeBase):
    def __init__(self, name, id = None, db = None,  idcatalog = 0):
        FstypeBase.__init__(self)
        self.catalog = None
        self.name = name      
        self.db = db
        self.idcatalog = idcatalog
        self.id = id


    
    def add(self, item): 
        self.items.append(item)
    
    def is_catalog(self): return True
    
    def get_icon(self): return icons["catalog"]
    
    def _load_data(self): 
        if self.db:
            return self.db.read_data(self.id) 
        else:
            return []
    
    def __setattr__(self, name, value):
        if name in ["name", "idcatalog"] and self.db and self.id and self.__dict__[name] != value:
            self.db.update("catalogs", self.id, name, value)    
        self.__dict__[name] = value
    
    def delete(self):
        for i in self.items:
            i.delete()
        self.db.delete("catalogs", self.id)
        self.db.commit()
   
    def get_catalog(self):
        if self.catalog: return self.catalog
        if self.idcatalog:
            d = self.db.get_catalog(self.idcatalog)[0]            
            self.catalog = Catalog(d["name"], d["id"], self.db, d["idcatalog"])                         
            return self.catalog
             
     

class Disk(FstypeBase):
    
    def __init__(self, name,  path, id = None, db = None, idcatalog = 0):
        FstypeBase.__init__(self)
        self.catalog = None
        self.name = name
        self.path = path
        self.db = db
        self.idcatalog = idcatalog  
        self.id = id

        if not db: self._data_loaded = True        
        
    def add(self, file):
        self.items.append(file)

    def is_disk(self): return True
   
    def get_icon(self): return icons["cdrom"]
    
    def __setattr__(self, name, value):
        if name in ["name", "idcatalog"] and self.id and self.__dict__[name] != value:
            self.db.update("disks", self.id, name, value)    
        self.__dict__[name] = value
    
    def _load_data(self): 
        return self.db.read_files_from_disk(self)
    
    def delete(self):
        for i in self.items:
            i.delete()
        self.db.delete("disks", self.id)
        self.db.commit()
        
      
    def get_catalog(self):
        if self.catalog: return self.catalog
        if self.idcatalog:
            d = self.db.get_catalog(self.idcatalog)[0]            
            self.catalog = Catalog(d["name"], d["id"], self.db, d["idcatalog"])                         
            return self.catalog
             
     
     
class File(FstypeBase):
    
    """ Acepta como información de bd o path """
    def __init__(self, info = None, disk = None, db = None): 
        FstypeBase.__init__(self)              
        self.__categories = None
        self.img = None        
        self.disk = disk
        
        if type(info) == str:
            self._data_loaded = True
            finfo = fileinfo.Info()
            
            self.path , self.name, self.size, self.type, self.mime, self.meta, self.img = finfo.get(info)
        else:

            self.name = info["name"]
            self.comment = info["comment"]            
            self.rating = info["rating"]        
            self.path = info["path"]
            self.size = info["size"]
            self.type = info["type"]
            self.mime = info["mime"]
            self.iddisk = info["iddisk"]
            self.db     = db
            self.id   = info["id"]
            
        
    def get_icon(self, size = 20): 
        if self.is_dir():
            return icons["directory"]
        type, subtype = self.mime.split("/")
        thumb = self.get_pixbuf()
        if thumb:
            return thumb
        
        if size == w and icons.has_key(self.mime): 
            return icons[self.mime]
        
        try:
            icon = icon_theme.load_icon("gnome-mime-" + type + "-" + subtype, size, 0)
            if size == w: icons[self.mime] = icon
            return icon
        except:            
            if icons.has_key(type): return icons[type]
        
        return icons["default"]
    
    def get_pixbuf(self):        
        img = self.get_thumbnail()
        if img:
            open('/tmp/tmp_gnomecatalog_icon.png', 'wb').write(img)
            icon =  gtk.gdk.pixbuf_new_from_file_at_size("/tmp/tmp_gnomecatalog_icon.png",80, 80)
            return icon
            
    def get_thumbnail(self):
        if self.id and not self.img:            
            self.img = self.db.get_thumbnail(self)
        return self.img
        
        
    def is_dir(self):
        return self.type == "directory"
        
    def is_file(self): 
        return self.type == "regular"
    
    def add_files(self, files): 
        self.items = files
    
    def __setattr__(self, name, value):
        if name in ["name", "comment", "rating"] and self.db and self.id != None and self.__dict__[name] != value:
            self.db.update("files", self.id, name, value)    
        self.__dict__[name] = value
        
    def get_disk(self):
        if self.disk: return self.disk        
        if self.iddisk:
            d = self.db.get_disk(self.iddisk)[0]
            self.disk = Disk( d["name"], d["root"], d["id"], self.db, d["idcatalog"])        
            return self.disk
 
           
    def _load_data(self): 
        return self.db.read_files_from_disk(self.disk, self.id)

        
    def delete(self):
        for i in self.items:
            i.delete()
        self.db.delete("files", self.id)     
        
    def get_location(self):
        disk = self.get_disk()
        item = disk
        txt = ""
        while True:
           item = item.get_catalog()
           if not item: break
           txt = txt + item.name + " / "
        txt = txt + disk.name 
        
        dir = os.path.dirname(self.path)[len(disk.path) + len("file:///")-1:]
        
        if dir: txt = txt + " - " + dir
        return txt

    
    def get_categories(self):
    	if self.__categories:
    		return self.__categories
        if self.id: 
        	self.__categories = self.db.get_categories_from_file(self.id)           
		return self.__categories


    def get_metadata(self):
        if self.id:
            return self.db.get_metadata_from_file(self.id)
