# -*- coding: UTF-8 -*-
from pysqlite2 import dbapi2 as pysqlite
import pysqlite2

#import sqlite
import string, os, pprint
import utils, fstypes

def dict_factory( cursor, row ):
    d = {}
    for idx, col in enumerate( cursor.description ):
        d[col[0]] = row[idx]
    return d

class DictCursor( pysqlite.Cursor ):
    def __init__( self, *args, **kwargs ):
        pysqlite.Cursor.__init__( self, *args, **kwargs )
        self.row_factory = dict_factory


class Sqlite:
    database = "gnomeCatalog.gcatalog"
    
    def __init__( self, dbfile = None, app = None ):
        self.database = dbfile
        self.app = app
        self.__deleted_data = False
        if( self.database ): 
            self.open( self.database)
        
    def open( self, database ):
        self.created = os.path.exists( database )            
        #self.db = sqlite.connect(database, encoding=('utf-8', 'ignore'))
        self.db = pysqlite.connect( database) #, isolation_level=None )
        
        self.cursor = self.db.cursor( factory=DictCursor )    

        if not self.created:
            self.__createDB()

                
    def sql( self, sql ):
        return self.fetchall( self.__sql( sql ) );

    def fetchall( self, result ):
        return self.cursor.fetchall();
            

        
    def __sql( self, sql ):
        #print "SQL: ",sql
        command = sql.split( " " )[0].lower()
        if command != "select":
            if command == "delete": self.__deleted_data = True
            
            self.app.set_guardar_state( True )
        try:
            return self.cursor.execute( sql )
        except: pass

    def commit( self ):
        self.db.commit()
        #self.__sql("COMMIT")
        
    def __createDB( self ):        
        self.__sql( "create table catalogs (id integer PRIMARY KEY, name varchar, idcatalog integer)" );        
        self.__sql( "create table disks (id integer PRIMARY KEY, name varchar, volname varchar, root varchar, idcatalog integer, rating integer)" );
        self.__sql( "create table files (id integer PRIMARY KEY, iddisk integer, idparent integer, name varchar, size integer, date timestamp, path varchar, mime varchar, type varchar, comment varchar, rating integer)" );
        self.__sql( "create table metadata (id integer, key varchar, value varchar)" );
        self.__sql( "create table thumbnails (id integer , img blob);" );    
        self.__sql( "create table categories (id integer PRIMARY KEY, name varchar);" );
        self.__sql( "create table rcategoriesfiles (idfile integer, idcategorie integer);" );        
        for c in [_("Movies"), _("Audio"), _("Pictures"), _("Documents")]:
            self.insert_category(c)
        self.created = True

    
    def _check( self, data ):
        
        txt = str(data)
        try:
            txt = txt.encode("utf-8", "replace")            
        except:            
            txt = unicode(txt, 'iso-8859-15').encode('utf-8')
    
        data = string.replace( txt, "'", "")
        return data

    def getFile( self, idfile ):
        metadata = self.sql( "select key, value from metadata where id=" + str( idfile ) )
        file = self.sql( "select files.*, thumbnails.img as img FROM files left outer join thumbnails on thumbnails.id=files.id where files.id=" + str( idfile ) )
        return [file, metadata]

    
    def set_categories_to_files(self, ids, categories):
        idstr = ", ".join( [str( i ) for i in ids] )
        self.sql( "DELETE  from rcategoriesfiles where idfile in (" + idstr +  ")" );
        for idfile in ids:
            for cat in categories:                
                self.sql( "insert into rcategoriesfiles (idfile, idcategorie) values (" + str( idfile ) + ", " + str( cat ) + ")" )
        
        self.commit()
                
        
    def get_categories( self ):
        result = self.sql( "select id, name from categories order by name" )
        res = []
        for col in result:
            res.append( [col["id"], col["name"]] )
        return res
        
    def get_categories_from_file(self, id ):
        result = self.sql( "select categories.*, rcategoriesfiles.idfile from categories left outer join rcategoriesfiles on categories.id=rcategoriesfiles.idcategorie and rcategoriesfiles.idfile=" + str( id ) );
        cats = []
        for col in result:
            try:
                if col["idfile"] != None:
                    cats.append(col["id"])
            except: pass
            
        return cats
    
    def get_metadata_from_file(self, id):
        meta = {}
        result = self.sql( "select key, value from metadata where id=" + str( id ) );        
        for item in result:
            meta[item["key"]] = item["value"]
        return meta
        
        
    def compress( self ):
        self.cursor.execute( "VACUUM" );
        
    
        
    def clear_thumbnails(self):
        self.__sql("DELETE FROM thumbnails")
        self.commit()
        
        
    def insert_category( self, cat ):
        self.__sql( "insert into categories values (Null, '" + cat + "')" )
        return self.cursor.lastrowid;
    
    
    
    """ inserta un disco en la bd """
    def __insert_disk(self, disk):
        self.__sql( "INSERT into disks (id, name, volname, root) values (NULL, '" + disk.name + "', '" +  self._check(disk.name)  + "', '" +  self._check(disk.path) + "')" );                    
        return self.cursor.lastrowid;
    
    """ Inserta los ficheros recursivamente """
    def __insert_files(self, iddisk, idparent, file):
        idparent = self.__insert_file(iddisk, idparent, file)
        
        if file.is_dir():
            for f in file:                
                idfile = self.__insert_files(iddisk, idparent, f)
    
    
    def __insert_file(self, iddisk, idparent, file):               
        self.__sql( "INSERT into files (id, iddisk, idparent, name, size, path, mime, type) values (NULL, " + str( iddisk ) +", " + str( idparent ) + " ,'" + self._check(file.name) + "', " + str( file.size ) + ", '" +  self._check(file.path) + "', '" + file.mime + "', '" + str( file.type ) + "')" )
        idfile =  self.cursor.lastrowid;

        """ Insetamos los metadatos del fichero """
        if file.meta:
            for key, val in file.meta.items():
                if key != "JPEGThumbnail":
                    self.__sql( "INSERT into metadata (id, key, value) values (" + str( idfile ) + ", '" + self._check( key ) + "', '" + self._check( val ) + "')" );
        
        thumb = file.get_thumbnail()
        if thumb:
            self.cursor.execute("INSERT INTO thumbnails VALUES(?, ?);",(idfile, buffer(thumb)))
                
        return idfile
        
        
    def insert_disk(self, disk):
        iddisk = self.__insert_disk(disk)
        for file in disk:
            self.__insert_files(iddisk, 0, file)                               
        self.db.commit()
        return fstypes.Disk(disk.name,  disk.path, iddisk, self)
    
    def insert_catalog(self, catalog): 
        self.__sql("INSERT INTO catalogs (id, name) values (NULL, '" + catalog.name + "')")
        catalog.id =  self.cursor.lastrowid;
        self.commit()
        return catalog.id
        
        
    def read_data(self, idparent = None):
        data = []
        if idparent:
            c_condition = " idcatalog = " + str(idparent)
        else:
            c_condition = "idcatalog=0 or idcatalog is null"
        
        catalogs = self.sql("select * from catalogs where " + c_condition)        
        for c in catalogs:
            data.append(fstypes.Catalog(c["name"],  c["id"], self, idcatalog = c["idcatalog"]))

        disks = self.sql("select * from disks where " + c_condition)
        for c in disks:
            data.append(fstypes.Disk(c["name"],  c["root"], c["id"], self, idcatalog = c["idcatalog"]))
        return data

        
    def read_files_from_disk(self, disk, idparent = None):
        data = []
        if idparent: c_condition = " idparent = " + str(idparent)
        else: c_condition = "idparent=0 or idparent is null"
        
        files = self.sql("select * from files where iddisk=" + str(disk.id) + " and " + c_condition + " order by type, id")
        for file in files:
            data.append(fstypes.File(file, disk, self))        
        return data

    def search_files(self, search, categorie = None, mime_type = None):
        data = []
        search_cond = ""
	if search: # search in metadata, comment and name
		search_cond = "  and (files.name like '%"  + search + "%' or files.comment like '%" + search + "%' or files.id in (select id from metadata where value like '%" + search + "%'))"
		
        if categorie:
	        sql = "select files.* from files where files.id in (select idfile from rcategoriesfiles where idcategorie='" + str(categorie)  + "') " + search_cond
	elif mime_type:
		sql = "select files.* from files where mime like '" + mime_type + "/%' " + search_cond
	else:
		sql = "select files.* from files where 1=1 " + search_cond

        files = self.sql(sql)
        for file in files:          
            data.append(fstypes.File(file, None, self))
        return data
        
    def update(self, type, id, name, value):
        self.__sql("update " + type +" set " + name + "='" + str( value ) + "' where id=" + str(id))
        self.commit()
        
    def delete(self, type, id):
        self.__sql("delete from " + type + " where id=" + str(id))
        
        if type == "categories":
            self.__sql("delete from rcategoriesfiles where idcategorie = " + str(id))
            
        if type == "files":
            self.__sql("delete from rcategoriesfiles where idfile = " + str(id))
            self.__sql("delete from metadata where id = " + str(id))
            self.__sql("delete from thumbnails where id= " + str(id))
            

    def get_thumbnail(self, file):
        rs = self.sql("select * from thumbnails where id=" + str(file.id))
        if len(rs) == 0: return None
        return rs[0]["img"]
    
    def get_disk(self, id):
        return self.sql("select * from disks where id=" + str(id))
    
    def get_catalog(self, id):        
        return self.sql("select * from catalogs where id=" + str(id))
        
    def has_deleted_data(self):
        return self.__deleted_data
    
    def get_categorie_id(self, name):
        result = self.sql("select id from categories where name ='" + name + "'")
        return result[0]["id"]
