# -*- coding: UTF-8 -*-
import sys, gtk, pprint, os
from gnomecatalog import config, utils, data

from gladeconnect import GladeConnect
import dialogs

class App(GladeConnect):
	
	def __init__(self, app_info):

 		GladeConnect.__init__(self, "gnomecatalog.glade")

		self.app_info = app_info

		self.config = config.Config()
		
	
		self.data = data.Data(self.treeviewDisk, self.treeviewFiles, self)
		self.toolbutton_add.set_menu(self.menu_toolbar_add);

		if len(sys.argv) > 1:
			database = sys.argv[1]
		else: 
			database  = self.config.get("database")
		
		self.data.open_database(database)
	
	 	self.restore_state()
	   	
	
	def save_state(self):
		"""Guarda el estado"""
		pos = self.mainApp.get_position()
		size = self.mainApp.get_size()
		self.config.save("width", size[0])		
		self.config.save("height", size[1])
		self.config.save("left", pos[0])		
		self.config.save("top", pos[1])
		
		
	def reload_categories(self):
		model = gtk.ListStore(str, str, bool)
		self.combobox_categories.set_model(model)
		
		model.append([_("Categories") + " / "+  _("File types"), "", False])
		model.append(["@@--types--@@", "@@--types--@@", True])
		for id, name in self.data.db.get_categories():
			model.append([name, id, False])
			
		self.combobox_categories.set_active(0)		
		model.append(["@@--types--@@", "@@--types--@@", True])
		self.combobox_categories.set_row_separator_func(self.__combo_categories)
		
		for key, name in {"audio" : _("Audio files"), "image" : _("Image files"), "text" : _("Text files"), "video" : _("Video files")}.items():
			model.append([name, key, True])
			
		
	def __combo_categories(self, model, iter):
		return model[iter][0] == "@@--types--@@"
		

		
	def get_selected_category(self):
		iter=  self.combobox_categories.get_active_iter()
		model =  self.combobox_categories.get_model()
		return [model[iter][1], model[iter][2]]

		#print model[iter][1]
		#self.data.treeFiles.filter(model[iter][1], model[iter][2])
		#if not self.combobox_categories.get_active() == 0:
		#	return self.combobox_categories.get_active_text()
		#return None
		
	def restore_state(self):		
		self.mainApp.move(int(self.config.get("left")), int(self.config.get("top")))
		self.mainApp.resize(int(self.config.get("width")), int(self.config.get("height")))		
		self.reload_categories()
		self.mainApp.show()
		                
		               
			
	#####################################
	# EVENTOS DE LA VENTANA PRINCIPAL
	#####################################


			
			
	def on_toolbutton_preferencias_clicked(self, window):
		dialogs.preferences(self.config)
				

	def on_combobox_categories_changed(self, widget): pass

		#iter=  self.combobox_categories.get_active_iter()
		#model =  self.combobox_categories.get_model()

		#print model[iter][1]
		#self.data.treeFiles.filter(model[iter][1], model[iter][2])

	#####################################
	# ACCIONES DEL TOOLBAR
	#####################################
		
	def on_toolbutton_add_clicked(self, window):
		dialogs.addDisk(self.data).loadDisk()

	def on_toolbutton_borrar_clicked(self, window):
		self.data.remove_selected()

	def on_toolbutton_buscar_clicked(self, window):
		if self.data.is_searching(): 
			self.data.cancel_action()
		else:
			self.data.search(self.entry_buscar.get_text(), self.get_selected_category())

		
	def on_entry_buscar_key_press_event(self, entry, event):

		if event.keyval == 65293:	# Pulsa enter
			self.on_toolbutton_buscar_clicked(entry);

	#####################################
	# ACCIONES SOBRE LOS TREEVIEWS
	#####################################
	# Pulsamos sobre el disco o directorio y hay que cargar los discos		
	def on_treeviewDisk_row_activated(self,widget, iter, path):
		self.data.show_files_from_selected()
		
	def on_treeviewDisk_cursor_changed(self, three):
		self.data.show_files_from_selected()
		
	# Esto es para precargar los directorios al abrirlos
	def on_treeviewDisk_row_expanded(self, widget, iter, path):
		self.data.preload_data(iter)

	# Se pulsa sobre la lista de ficheros.			
	def on_treeviewFiles_row_activated(self, widget, iter, column): pass
		#print widget, iter, column
		#self.data.clickOnFiles(iter)

	
	#####################################
	# ACCIONES DEL MENU
	#####################################
	def on_menu_abrir_activate(self, widget):
		self.data.open_database(dialogs.FileDialogs().open())
		
	def on_menu_nuevo_activate(self, widget):
		self.data.open_database(dialogs.FileDialogs().new())
   						
	def on_menu_nuevo_catalogo_activate(self, widget):
		self.data.new_catalog()


	def on_menu_compactar_activate(self, window):
		self.data.db.compress()
	def on_treeviewFiles_popup_menu(self, window, event):
		if event.type == gtk.gdk._2BUTTON_PRESS:			
			self.data.click_on_fileview()
			
		if event.button == 3:	# Si se pulsa el boton derecho
			self.menu_files.popup(None,None,None,event.button,  event.time)		
		
		self.data.treeFiles.check_star_clicked(event)
		
	def on_treeviewDisk_key_press_event(self, entry, event):
		if event.keyval == 65471: # f2 key
			self.data.treeDisk.edit()

	def on_treeviewFiles_key_press_event(self, entry, event):
		if event.keyval == 65471: # f2 key
			self.data.treeFiles.edit()

	def on_mainApp_button_press_event(self, entry, event): pass
		
	def on_renombrar_activate(self, widget):	
		self.data.treeFiles.edit()
		
	def on_menu_edit_comments_categories_activate(self, window):
		self.data.show_properties_dialog()
		
	def	on_menu_info_activate(self, window):
		self.data.show_information_dialog()

	def on_menu_guardar_activate(self, window):
		self.data.save_database(False)

	def on_guardar_como_activate(self, window):
		self.data.database_filename, clear_thumbnails = dialogs.FileDialogs().save_as()
		self.data.save_database(clear_thumbnails)
		self.data.open_database(self.data.database_filename)
   
	def on_menu_acercade_activate(self, window):
		dialogs.acercaDe(self.app_info)
				
	def on_menu_import_activate(self, widget):
		dialogs.Import(self.config)
		
	def on_menu_export_activate(self, widget):
		dialogs.Export(self.config)

	def on_menu_quit_activate(self, window, other=None):
		if not self.estado_guardado:
			dialogs.quitDialog(self)		
			return True
		else:
			self.quit()
			
	
	def quit(self):
		
		self.save_state()
		gtk.main_quit()
		
	
	def set_guardar_state(self, state):
		self.estado_guardado = not state
		self.toolbutton_guardar.set_sensitive(state)
		self.menu_guardar.set_sensitive(state)
			


